/*****************************************************************************************
 *
 * main.c
 * Copyright (c) 2010 NuukiaWorld
 *
 *  This file is part of "Network database".
 *
 *  "Network database" is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Network database" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with "Network database".  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        31.01.2010    NuukiaWorld        Initial Release
 *
 ****************************************************************************************/

#include "main.h"

/**************************************************************************************/

/* expose preprocessor constants to Rexx */
REXX_EXPOSE(MSG_OPEN_STATIC_MENU);
REXX_EXPOSE(DF3_CURSOR_MOVABLE);
REXX_EXPOSE(DF3_FLOAT_NUMBER);

REXX_EXPOSE(OMSG_CURRENT_NETWORK);
REXX_EXPOSE(OMSG_COUNTRY_LIST);
REXX_EXPOSE(OMSG_NAME_DIALOG);
REXX_EXPOSE(OMSG_MCCMNC_DIALOG);

REXX_EXPOSE(MAXLEN_NAME);

/**************************************************************************************/

extern const struct menu menu_main;

extern const struct dialog dlg_info;

extern const nokstr_t str_not_found;
extern const nokstr_t str_networks_found;

/**************************************************************************************/

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint main_dispatcher_id; /* dispatcher's call id */
rexx("main_dispatcher_id=add_5e0_id(main_dispatcher)"); /* auto creation of the id */

unsigned main_dispatcher(unsigned msg)
{
   struct location loc;
   unsigned int n, mnc, ext, pos;
   char *s;
   unsigned char mcc[2];

   switch(msg)
   {
      case MSG_D_INIT: // application initialization
         app.dlg = NULL;
         app.name[0] = app.mccmnc[0] = '\0';
         menu_create( 1, MDM_LIST | MDM_SUB, MCM_MARK_ID | MCM_OWN_COUNTER, 1, NULL );

         // open main menu
         send_message( MSG_OPEN_STATIC_MENU | MSG_1P, &menu_main );
         msg = MSG_R_STAY;
         break;

      case MSG_D_QUIT: // application cleanup
         clear_menu();
         menu_delete(1);
         msg = MSG_R_QUIT;
         break;

      case MSG_BACK:
         if( app.dlg == NULL ) { // menu cancel
            clear_menu();
         } else {                // dialog cancel
            dialog_close();
            menu_reopen();
            app.dlg = NULL;
            msg = MSG_NOP;
         }
         break;

      case OMSG_CURRENT_NETWORK:
         own_get_location(&loc);
         send_message( OMSG_NETWORK_INFO | MSG_1P, to_mccmnc( loc.mcc, loc.mnc, 0x0F ) );
         msg = MSG_NOP;
         break;

      case OMSG_COUNTRY_LIST:
         n = list_countries( (char*)&msg_argv[0] );
         if( n != 0 ) {
            menu_open( 1, 1 );
         } else {
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION, str_not_found );
            menu_reopen();
         }
         msg = MSG_NOP;
         break;

      case OMSG_NETWORK_LIST:
         clear_menu();   // Destroy country list to avoid too many nested menus
         pos = ( msg_argv[0] & 0xFFFF );
         n = list_networks( (unsigned char*)&msg_argv[0], pos );
         if( n != 0 ) {
            if( pos == 0 ) {
               send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION,
                  (const nokstr_t*)w_sprintf( &str_networks_found, n ) );
            }
            menu_open( 1, 1 );
         } else {
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION, &str_not_found );
            menu_reopen();
         }
         msg = MSG_NOP;
         break;

      case OMSG_NETWORK_INFO:
         s = (char*)create_network_info( (const unsigned char*)&msg_argv[0] );
         if( s != NULL ) {
            dialog_open( s, app.dlg = &dlg_info );
         } else {
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION, &str_not_found );
            menu_reopen();
         }
         msg = MSG_NOP;
         break;

      case OMSG_NAME_DIALOG:
         send_message( MSG_ALPHANUMERIC_MODE );
         dialog_open_msg( app.name, app.dlg = (const struct dialog*)msg_argv[0], OMSG_FIND_NAME );
         msg = MSG_NOP;
         break;

      case OMSG_FIND_NAME:
         app.dlg = NULL;
         n = find_name(0);
         if( n != 0 ) {
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION,
               (const nokstr_t*)w_sprintf( &str_networks_found, n ) );
            menu_open( 1, 1 );
         } else {
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION, str_not_found );
            menu_reopen();
         }
         msg = MSG_NOP;
         break;
         
      case OMSG_FIND_MORE:
         clear_menu();
         find_name(msg_argv[0]);
         menu_open( 1, 1 );
         msg = MSG_NOP;
         break;

      case OMSG_MCCMNC_DIALOG:
         send_message( MSG_NUMERIC_MODE );
         dialog_open_msg( app.mccmnc, app.dlg = (const struct dialog*)msg_argv[0], OMSG_FIND_MCCMNC );
         msg = MSG_NOP;
         break;

      case OMSG_FIND_MCCMNC:
         app.dlg = NULL;
         s = strchr( app.mccmnc, '.' );
         if( s != NULL ) *s = '\0';       // Remove the dot
         
         n = own_atol_hex(app.mccmnc);
         if( ( n >> 12 ) != 0 ) n = 0;    // Too many significant digits for MCC
         mcc[0] = ( ( n >> 8 ) & 0x0F ) | ( n & 0xF0 );
         mcc[1] = (unsigned char)( n | 0xF0 );
         
         if( s == NULL ) {                // only MCC
            send_message( OMSG_NETWORK_LIST | MSG_1P, to_mccmnc( mcc, 0, 0 ) );
         } else {                         // both MCC.MNC
            *s++ = '.';                   // Restore the dot
            n = own_atol_hex(s);
            ext = 0x0F;
            if( ( n >> 8 ) != 0 ) {       // 3-digit (extended) MNC
               ext = n;
               n >>= 4;
            }
            mnc = ( ( n >> 4 ) & 0x0F ) | ( ( n << 4 ) & 0xF0 );
            send_message( OMSG_NETWORK_INFO | MSG_1P, to_mccmnc( mcc, mnc, ext ) );
         }
         msg = MSG_NOP;
         break;
   }
   return msg;
}

/**************************************************************************************/

