#ifndef MAIN_H
#define MAIN_H

/*****************************************************************************************
 *
 * main.h
 *
 * Part of "Network Database"
 * Copyright (c) 2010 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        31.01.2010    NuukiaWorld        Initial Release
 *
 ****************************************************************************************/

#include <nokix/all.h>

/**************************************************************************************/

#define  MAXLEN_NAME 16       // maximum search text length
#define  MAXLEN_MENU 50       // maximum number of networks per menu (avoid out-of-memory hang)

/* KEEP THIS STRUCTURE <=40 BYTES LONG! */
struct application
{
   const struct dialog  *dlg;     // currently active dialog
   char                 name[MAXLEN_NAME + 1];
   char                 mccmnc[8];
};

/* global_buffer is an array of 40 bytes in RAM which you can use for your
   own purposes as long as your application stays active; when it is deactivated
   the buffer may be overwritten by another one;
   here we define 'app' symbol to access these 40 bytes as a structure */
#define app (*(struct application *)global_buffer)

extern   void           clear_menu(void);

extern   unsigned int   to_mccmnc( const unsigned char mcc[2], unsigned int mnc, unsigned int extend );

extern   unsigned int   list_countries( const char range[2] );
extern   unsigned int   list_networks( const unsigned char mcc[2], unsigned int first );
extern   nokstr_t       *create_network_info( const unsigned char mccmnc[4] );
extern   unsigned int   find_name( unsigned int first );

#define OMSG_CURRENT_NETWORK  (0x1f00) // Show information for current network
#define OMSG_COUNTRY_LIST     (0x1f01) // Open country list (arg specifies country range)
#define OMSG_NETWORK_LIST     (0x1f02) // List networks for one country, arg=MCC array & position
#define OMSG_NETWORK_INFO     (0x1f03) // Show information for one network, arg=MCC/MNC array
#define OMSG_NAME_DIALOG      (0x1f04) // Open name search dialog
#define OMSG_FIND_NAME        (0x1f05) // Perform name search for app.buf
#define OMSG_FIND_MORE        (0x1f06) // Continue name search for app.buf, arg=position
#define OMSG_MCCMNC_DIALOG    (0x1f07) // Open MCC.MNC search dialog
#define OMSG_FIND_MCCMNC      (0x1f08) // Perform MCC.MNC search for app.buf

/**************************************************************************************/

#endif /* MAIN_H */

