/*****************************************************************************************
 *
 * plmn.c
 * Copyright (c) 2010 NuukiaWorld
 *
 *  This file is part of "Network database".
 *
 *  "Network database" is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Network database" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with "Network database".  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        31.01.2010    NuukiaWorld        Initial Release
 *  1.0.1      24.09.2010    NuukiaWorld        Unicode caption displayed only when needed
 *
 ****************************************************************************************/

#include "main.h"
#include <ctype.h>

/**************************************************************************************/

/**************************************************************************************/

extern const nokstr_t str_more;
extern const nokstr_t str_network_info;

/**************************************************************************************/

static char *format_mcc( char *buf, const struct ppm_plmn *cntr ) {
   *buf++ = '0' + ( cntr->mcc[0] & 0x0F );
   *buf++ = '0' + ( cntr->mcc[0] >> 4 );
   *buf++ = '0' + ( cntr->mcc[1] & 0x0F );
   *buf = '\0';
   return buf;
}

static char *format_country( char *buf, const struct ppm_plmn *cntr ) {
   strcpy( buf, cntr->country );
   buf = strchr( buf, '\0' );
   *buf++ = ' ';
   return buf; // Note: no NUL at end
}

static char *format_mnc( char *buf, const struct ppm_plmn_mnc *nwrk ) {
   *buf++ = '0' + ( nwrk->mnc & 0x0F );
   *buf++ = '0' + ( nwrk->mnc >> 4 );
   if( ( nwrk->extend & 0xF0 ) != 0xF0 ) *buf++ = '0' + ( nwrk->extend >> 4 );
   *buf = '\0';
   return buf;
}

static void format_name( char *buf, const struct ppm_plmn_mnc *nwrk ) {
   strncpy( buf, nwrk->name + ( nwrk->extend & 0x0F ) * 2, nwrk->namelen );   // skip the Unicode
   buf[nwrk->namelen] = '\0';
}

static char *strcasestr( const char *s1, const char *s2 ) {
	const char *t1, *t2;

	do {
		t1 = s1, t2 = s2;
		do {
			if( *t2 == '\0' ) return (char*)s1;
		} while( toupper(*t1++) == toupper(*t2++) );
	} while( *s1++ != '\0' );
	return NULL;
}

static unsigned int menu_add_item_sorted( const struct menu_item *item ) {
   const struct menu_item *item2;
   unsigned int pos;

   for( pos = 1; ( item2 = menu_get_item( 1, pos ) ) != NULL
      && aord_stricmp( (char*)item->title, (char*)item2->title ) >= 0 ; pos++ );
   return menu_add_item( 1, pos, item );
}

static void add_result( const struct ppm_plmn *cntr, const struct ppm_plmn_mnc *nwrk,
   struct menu_item *item ) {
   
   char *title;

   title = malloc_try( 8 + nwrk->namelen + 1 ); // "CNT MNC name\0"
   if( title == NULL ) return;                  // out of memory

   item->title = (nokstr_t*)title;
   title = format_country( title, cntr );
   title = format_mnc( title, nwrk );
   *title++ = ' ';
   format_name( title, nwrk );

   item->arg = to_mccmnc( cntr->mcc, nwrk->mnc, nwrk->extend >> 4 );
   item->msg = OMSG_NETWORK_INFO;
   item->flags = MIF_FINISH_SHORTCUT;
   menu_add_item_sorted(item);
   // TODO: detect menu_add_item() failure and free item.title immediately!
}

static void add_more( unsigned int msg, unsigned int arg, struct menu_item *item ) {
   item->msg = msg;
   item->arg = arg;
   item->title = (nokstr_t*)&str_more;
   item->flags = MIF_LEAVE_MENU;
   menu_add_item( 1, 0, item );
}

static const struct ppm_plmn *find_country( const unsigned char mcc[2] ) {
   struct ppm_header *ppm = NULL;
   struct ppm_subchunk_header *sub = NULL;
   struct ppm_plmn *cntr = NULL;

   while( ( cntr = (struct ppm_plmn*)find_ppm_subchunk( &ppm, &sub, PPM_PLMN,
      (struct ppm_subchunk_header*)cntr, PPM_NEXT, 0 ) ) != NULL && cntr->cntr != 0 ) {

      if( cntr->mcc[0] == mcc[0] && cntr->mcc[1] == mcc[1] ) return cntr;
   }
   return NULL;
}

static const struct ppm_plmn_mnc *find_network( const struct ppm_plmn *cntr, const unsigned char mnc[2] ) {
   const struct ppm_plmn_mnc *nwrk;
   unsigned int c;

   nwrk = cntr->mnc;
   for( c = cntr->count; c > 0; c-- ) {
      if( nwrk->mnc == mnc[0] && ( nwrk->extend >> 4 ) == mnc[1] ) return nwrk;
      nwrk = (const struct ppm_plmn_mnc*)( (char*)nwrk + nwrk->length );
   }
   return NULL;
}

/**************************************************************************************/

/* Deletes all dynamic menu items freeing the memory reserved for each title
*/
void clear_menu(void) {
   struct menu_item *item;

   while( ( item = menu_get_item( 1, 1 ) ) != NULL ) {
      free( item->title );
      menu_rem_item( 1, 1 );
   }
}

/* Packs BCD MCC and MNC values in one integer, returns integer with bits
   31...24     first byte of BCD MCC
   23...16     second byte of BCD MCC (upper nibble must be 0xF)
   15...8      BCD MNC
   7...4       zero
   3...0       extended MNC digit or 0xF
*/
unsigned int to_mccmnc( const unsigned char mcc[2], unsigned int mnc, unsigned int extend ) {
   return ( mcc[0] << 24 ) | ( mcc[1] << 16 ) | ( mnc << 8 ) | ( extend & 0x0F );
}

/* Fills the dynamic menu with countries in the specified alphabet range, returns the count
   range[0]    first letter of alphabet to include ( '\x00'...'\x3F' mapped to '\x40'...'\x7F')
   range[1]    last letter of alphabet to include ( '\x00'...'\x3F' mapped to '\x40'...'\x7F')
*/
unsigned int list_countries( const char range[2] ) {
   /*const*/ struct ppm_header *ppm = NULL;
   /*const*/ struct ppm_subchunk_header *sub = NULL;
   /*const*/ struct ppm_plmn *cntr = NULL;
   unsigned int count = 0;
   struct menu_item item;
   char *title;

   menu_init_item(&item);
   item.msg = OMSG_NETWORK_LIST;
   item.flags = MIF_LEAVE_MENU;

   while( ( cntr = (struct ppm_plmn*)find_ppm_subchunk( &ppm, &sub, PPM_PLMN,
      (struct ppm_subchunk_header*)cntr, PPM_NEXT, 0 ) ) != NULL && cntr->cntr != 0 ) {

      if( ( cntr->country[0] | '\x40' ) >= range[0] && ( cntr->country[0] | '\x40' ) <= range[1] ) {
         title = malloc_try( 9 + 1 );  // "CNT (MNC)\0"
         if( title == NULL ) break;    // out of memory
         
         item.title = (nokstr_t*)title;
         title = format_country( title, cntr );
         *title++ = '(';
         title = format_mcc( title, cntr );
         *title++ = ')';
         *title = '\0';
         
         item.arg = to_mccmnc( cntr->mcc, 0, 0 );
         menu_add_item_sorted(&item);
         // TODO: detect menu_add_item() failure and free item.title immediately!
         count++;
      }
   }
   return count;
}

/* Fills the dynamic menu with networks in the specified country, returns the total count
   mcc[0]   first byte of BCD MCC
   mcc[1]   second byte of BCD MCC (upper nibble must be 0xF)
   first    first match to include in the menu (for multi-page menu)
*/
unsigned int list_networks( const unsigned char mcc[2], unsigned int first ) {
   const struct ppm_plmn *cntr;
   const struct ppm_plmn_mnc *nwrk;
   unsigned int count;
   struct menu_item item;

   cntr = find_country(mcc);
   if( cntr == NULL ) return 0;

   menu_init_item(&item);

   nwrk = cntr->mnc;
   for( count = 0; count < cntr->count; count++ ) {
      if( count >= first && count < first + MAXLEN_MENU ) add_result( cntr, nwrk, &item );
      if( count == first + MAXLEN_MENU ) {
         add_more( OMSG_NETWORK_LIST, ( *(unsigned short*)mcc << 16 ) + count, &item);
      }
      nwrk = (const struct ppm_plmn_mnc*)( (char*)nwrk + nwrk->length );
   }
   return count;
}

/* Returns information string for network that matches the given MCC and MNC, NULL if not found
   mcc_mnc[0]  first byte of BCD MCC
   mcc_mnc[1]  second byte of BCD MCC (upper nibble must be 0xF)
   mcc_mnc[2]  BCD MNC
   mcc_mnc[3]  extended MNC digit or 0x0F
*/
nokstr_t *create_network_info( const unsigned char mccmnc[4] ) {
   const struct ppm_plmn *cntr;
   const struct ppm_plmn_mnc *nwrk;
   char mcc_buf[4], mnc_buf[4], name_buf[MAXLEN_NAME + 1];
   nokstr_t *s;
   unsigned int unicode_len;
   wchar_t *p;

   cntr = find_country(mccmnc);
   if( cntr == NULL ) return NULL;
   nwrk = find_network( cntr, mccmnc + 2 );
   if( nwrk == NULL ) return NULL;

   format_mcc( mcc_buf, cntr );
   format_mnc( mnc_buf, nwrk );
   format_name( name_buf, nwrk );
   s = w_sprintf( &str_network_info, mcc_buf, mnc_buf, cntr->country, name_buf );
   
   unicode_len = ( nwrk->extend & 0x0F );
   if( unicode_len != 0 ) return (nokstr_t*)wcsncat( (wchar_t*)s, (wchar_t*)nwrk->name, unicode_len );
   // Remove caption "Unicode:"
   for( p = wcschr( (wchar_t*)s, L'\0' ); *--p != L'\n'; ); // Find last line feed
   *p = L'\0';                                              // End string there
   return s;
}

/* Fill the dynamic menu with networks matching the specified text in the name, returns the count
   first    first match to include in the menu (for multi-page menu)
*/
unsigned int find_name( unsigned int first ) {
   /*const*/ struct ppm_header *ppm = NULL;
   /*const*/ struct ppm_subchunk_header *sub = NULL;
   /*const*/ struct ppm_plmn *cntr = NULL;
   const struct ppm_plmn_mnc *nwrk;
   unsigned int count = 0, c;
   struct menu_item item;
   char name_buf[MAXLEN_NAME + 1];

   menu_init_item(&item);

   while( ( cntr = (struct ppm_plmn*)find_ppm_subchunk( &ppm, &sub, PPM_PLMN,
      (struct ppm_subchunk_header*)cntr, PPM_NEXT, 0 ) ) != NULL && cntr->cntr != 0 ) {

      nwrk = cntr->mnc;
      for( c = cntr->count; c > 0; c-- ) {      // search all networks
         format_name( name_buf, nwrk );
         if( strcasestr( name_buf, app.name ) != NULL ) {
            if( count >= first && count < first + MAXLEN_MENU ) add_result( cntr, nwrk, &item );
            if( count == first + MAXLEN_MENU ) add_more( OMSG_FIND_MORE, count, &item);
            count++;
         }
         nwrk = (const struct ppm_plmn_mnc*)( (char*)nwrk + nwrk->length );
      }
   }
   return count;
}

